@@ -5,25 +5,36 @@ import traceback
 from collections import defaultdict
 from enum import Enum
 from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Tuple, Union
+from urllib.parse import urlparse
 
 import click
 import hypothesis
 import yaml
 
 from .. import checks as checks_module
+from .. import fixups as _fixups
 from .. import runner
 from .. import targets as targets_module
-from ..constants import DEFAULT_DATA_GENERATION_METHODS, DEFAULT_STATEFUL_RECURSION_LIMIT, DataGenerationMethod
+from ..constants import (
+    DEFAULT_DATA_GENERATION_METHODS,
+    DEFAULT_STATEFUL_RECURSION_LIMIT,
+    CodeSampleStyle,
+    DataGenerationMethod,
+)
 from ..fixups import ALL_FIXUPS
 from ..hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher, HookScope
 from ..models import CheckFunction
-from ..runner import events
+from ..runner import events, prepare_hypothesis_settings
+from ..schemas import BaseSchema
+from ..specs.openapi import loaders as oas_loaders
 from ..stateful import Stateful
 from ..targets import Target
 from ..types import Filter
+from ..utils import file_exists, get_requests_auth, import_app
 from . import callbacks, cassettes, output
 from .constants import DEFAULT_WORKERS, MAX_WORKERS, MIN_WORKERS
 from .context import ExecutionContext
+from .debug import DebugOutputHandler
 from .handlers import EventHandler
 from .junitxml import JunitXMLHandler
 from .options import CSVOption, CustomHelpMessageChoice, NotSet, OptionalInt
@@ -330,6 +341,11 @@ class GroupedOption(click.Option):
 @click.option(
     "--junit-xml", help="Create junit-xml style report file at given path.", type=click.File("w", encoding="utf-8")
 )
+@click.option(
+    "--debug-output-file",
+    help="Save debug output as JSON lines in the given file.",
+    type=click.File("w", encoding="utf-8"),
+)
 @click.option(
     "--show-errors-tracebacks",
     help="Show full tracebacks for internal errors.",
@@ -338,6 +354,13 @@ class GroupedOption(click.Option):
     default=False,
     show_default=True,
 )
+@click.option(
+    "--code-sample-style",
+    help="Controls the style of code samples for failure reproduction.",
+    type=click.Choice([item.name for item in CodeSampleStyle]),
+    default=CodeSampleStyle.default().name,
+    callback=callbacks.convert_code_sample_style,
+)
 @click.option(
     "--store-network-log", help="Store requests and responses into a file.", type=click.File("w", encoding="utf-8")
 )
@@ -427,8 +450,11 @@ class GroupedOption(click.Option):
     cls=GroupedOption,
     group=ParameterGroup.hypothesis,
 )
+@click.option("--no-color", help="Disable ANSI color escape codes.", type=bool, is_flag=True)
 @click.option("--verbosity", "-v", help="Reduce verbosity of error output.", count=True)
+@click.pass_context
 def run(
+    ctx: click.Context,
     schema: str,
     auth: Optional[Tuple[str, str]],
     auth_type: str,
@@ -451,7 +477,9 @@ def run(
     validate_schema: bool = True,
     skip_deprecated_operations: bool = False,
     junit_xml: Optional[click.utils.LazyFile] = None,
+    debug_output_file: Optional[click.utils.LazyFile] = None,
     show_errors_tracebacks: bool = False,
+    code_sample_style: CodeSampleStyle = CodeSampleStyle.default(),
     store_network_log: Optional[click.utils.LazyFile] = None,
     fixups: Tuple[str] = (),  # type: ignore
     stateful: Optional[Stateful] = None,
@@ -466,12 +494,14 @@ def run(
     hypothesis_seed: Optional[int] = None,
     hypothesis_verbosity: Optional[hypothesis.Verbosity] = None,
     verbosity: int = 0,
+    no_color: bool = False,
 ) -> None:
     """Perform schemathesis test against an API specified by SCHEMA.
 
     SCHEMA must be a valid URL or file path pointing to an Open API / Swagger specification.
     """
     # pylint: disable=too-many-locals
+    maybe_disable_color(ctx, no_color)
     check_auth(auth, headers)
     selected_targets = tuple(target for target in targets_module.ALL_TARGETS if target.__name__ in targets)
 
@@ -480,47 +510,235 @@ def run(
     else:
         selected_checks = tuple(check for check in checks_module.ALL_CHECKS if check.__name__ in checks)
 
-    prepared_runner = runner.prepare(
+    if fixups:
+        if "all" in fixups:
+            _fixups.install()
+        else:
+            _fixups.install(fixups)
+    hypothesis_settings = prepare_hypothesis_settings(
+        deadline=hypothesis_deadline,
+        derandomize=hypothesis_derandomize,
+        max_examples=hypothesis_max_examples,
+        phases=hypothesis_phases,
+        report_multiple_bugs=hypothesis_report_multiple_bugs,
+        suppress_health_check=hypothesis_suppress_health_check,
+        verbosity=hypothesis_verbosity,
+    )
+    event_stream = into_event_stream(
         schema,
+        app=app,
+        base_url=base_url,
+        validate_schema=validate_schema,
+        skip_deprecated_operations=skip_deprecated_operations,
+        data_generation_methods=data_generation_methods,
+        force_schema_version=force_schema_version,
+        request_tls_verify=request_tls_verify,
         auth=auth,
         auth_type=auth_type,
         headers=headers,
+        endpoint=endpoints or None,
+        method=methods or None,
+        tag=tags or None,
+        operation_id=operation_ids or None,
         request_timeout=request_timeout,
-        request_tls_verify=request_tls_verify,
-        base_url=base_url,
-        endpoint=endpoints,
-        method=methods,
-        tag=tags,
-        operation_id=operation_ids,
-        app=app,
         seed=hypothesis_seed,
         exit_first=exit_first,
         dry_run=dry_run,
         store_interactions=store_network_log is not None,
         checks=selected_checks,
-        data_generation_methods=data_generation_methods,
         max_response_time=max_response_time,
         targets=selected_targets,
         workers_num=workers_num,
-        validate_schema=validate_schema,
-        skip_deprecated_operations=skip_deprecated_operations,
-        fixups=fixups,
         stateful=stateful,
         stateful_recursion_limit=stateful_recursion_limit,
-        force_schema_version=force_schema_version,
-        hypothesis_deadline=hypothesis_deadline,
-        hypothesis_derandomize=hypothesis_derandomize,
-        hypothesis_max_examples=hypothesis_max_examples,
-        hypothesis_phases=hypothesis_phases,
-        hypothesis_report_multiple_bugs=hypothesis_report_multiple_bugs,
-        hypothesis_suppress_health_check=hypothesis_suppress_health_check,
-        hypothesis_verbosity=hypothesis_verbosity,
+        hypothesis_settings=hypothesis_settings,
     )
     execute(
-        prepared_runner, workers_num, show_errors_tracebacks, validate_schema, store_network_log, junit_xml, verbosity
+        event_stream,
+        workers_num,
+        show_errors_tracebacks,
+        validate_schema,
+        store_network_log,
+        junit_xml,
+        verbosity,
+        code_sample_style,
+        debug_output_file,
     )
 
 
+def into_event_stream(
+    schema_location: str,
+    *,
+    app: Any,
+    base_url: Optional[str],
+    validate_schema: bool,
+    skip_deprecated_operations: bool,
+    data_generation_methods: Tuple[DataGenerationMethod, ...],
+    force_schema_version: Optional[str],
+    request_tls_verify: Union[bool, str],
+    # Network request parameters
+    auth: Optional[Tuple[str, str]],
+    auth_type: Optional[str],
+    headers: Optional[Dict[str, str]],
+    request_timeout: Optional[int],
+    # Schema filters
+    endpoint: Optional[Filter],
+    method: Optional[Filter],
+    tag: Optional[Filter],
+    operation_id: Optional[Filter],
+    # Runtime behavior
+    checks: Iterable[CheckFunction],
+    max_response_time: Optional[int],
+    targets: Iterable[Target],
+    workers_num: int,
+    hypothesis_settings: Optional[hypothesis.settings],
+    seed: Optional[int],
+    exit_first: bool,
+    dry_run: bool,
+    store_interactions: bool,
+    stateful: Optional[Stateful],
+    stateful_recursion_limit: int,
+) -> Generator[events.ExecutionEvent, None, None]:
+    try:
+        if app is not None:
+            app = import_app(app)
+        loaded_schema = load_schema(
+            schema_location,
+            app=app,
+            base_url=base_url,
+            validate_schema=validate_schema,
+            skip_deprecated_operations=skip_deprecated_operations,
+            data_generation_methods=data_generation_methods,
+            force_schema_version=force_schema_version,
+            request_tls_verify=request_tls_verify,
+            auth=auth,
+            auth_type=auth_type,
+            headers=headers,
+            endpoint=endpoint or None,
+            method=method or None,
+            tag=tag or None,
+            operation_id=operation_id or None,
+        )
+        yield from runner.from_schema(
+            loaded_schema,
+            auth=auth,
+            auth_type=auth_type,
+            headers=headers,
+            request_timeout=request_timeout,
+            request_tls_verify=request_tls_verify,
+            seed=seed,
+            exit_first=exit_first,
+            dry_run=dry_run,
+            store_interactions=store_interactions,
+            checks=checks,
+            max_response_time=max_response_time,
+            targets=targets,
+            workers_num=workers_num,
+            stateful=stateful,
+            stateful_recursion_limit=stateful_recursion_limit,
+            hypothesis_settings=hypothesis_settings,
+        ).execute()
+    except Exception as exc:
+        yield events.InternalError.from_exc(exc)
+
+
+def load_schema(
+    schema_location: str,
+    *,
+    app: Any,
+    base_url: Optional[str],
+    validate_schema: bool,
+    skip_deprecated_operations: bool,
+    data_generation_methods: Tuple[DataGenerationMethod, ...],
+    force_schema_version: Optional[str],
+    request_tls_verify: Union[bool, str],
+    # Network request parameters
+    auth: Optional[Tuple[str, str]],
+    auth_type: Optional[str],
+    headers: Optional[Dict[str, str]],
+    # Schema filters
+    endpoint: Optional[Filter],
+    method: Optional[Filter],
+    tag: Optional[Filter],
+    operation_id: Optional[Filter],
+) -> BaseSchema:
+    """Automatically load API schema."""
+    loader = detect_loader(schema_location, app)
+    kwargs = get_loader_kwargs(
+        loader,
+        app=app,
+        base_url=base_url,
+        validate_schema=validate_schema,
+        skip_deprecated_operations=skip_deprecated_operations,
+        data_generation_methods=data_generation_methods,
+        force_schema_version=force_schema_version,
+        request_tls_verify=request_tls_verify,
+        auth=auth,
+        auth_type=auth_type,
+        headers=headers,
+        endpoint=endpoint,
+        method=method,
+        tag=tag,
+        operation_id=operation_id,
+    )
+    return loader(schema_location, **kwargs)
+
+
+def detect_loader(schema_location: str, app: Any) -> Callable:
+    """Detect API schema loader."""
+    if file_exists(schema_location):
+        # If there is an existing file with the given name,
+        # then it is likely that the user wants to load API schema from there
+        return oas_loaders.from_path
+    if app is not None and not urlparse(schema_location).netloc:
+        # App is passed & location is relative
+        return oas_loaders.get_loader_for_app(app)
+    # Default behavior
+    return oas_loaders.from_uri
+
+
+def get_loader_kwargs(
+    loader: Callable,
+    *,
+    app: Any,
+    base_url: Optional[str],
+    endpoint: Optional[Filter],
+    method: Optional[Filter],
+    tag: Optional[Filter],
+    operation_id: Optional[Filter],
+    skip_deprecated_operations: bool,
+    validate_schema: bool,
+    force_schema_version: Optional[str],
+    data_generation_methods: Tuple[DataGenerationMethod, ...],
+    # Network request parameters
+    auth: Optional[Tuple[str, str]],
+    auth_type: Optional[str],
+    headers: Optional[Dict[str, str]],
+    request_tls_verify: Union[bool, str],
+) -> Dict[str, Any]:
+    """Detect the proper set of parameters for a loader."""
+    # These kwargs are shared by all loaders
+    kwargs = {
+        "app": app,
+        "base_url": base_url,
+        "method": method,
+        "endpoint": endpoint,
+        "tag": tag,
+        "operation_id": operation_id,
+        "skip_deprecated_operations": skip_deprecated_operations,
+        "validate_schema": validate_schema,
+        "force_schema_version": force_schema_version,
+        "data_generation_methods": data_generation_methods,
+    }
+    if loader is not oas_loaders.from_path:
+        kwargs["headers"] = headers
+    if loader in (oas_loaders.from_uri, oas_loaders.from_aiohttp):
+        kwargs["verify"] = request_tls_verify
+        if auth is not None:
+            kwargs["auth"] = get_requests_auth(auth, auth_type)
+    return kwargs
+
+
 def check_auth(auth: Optional[Tuple[str, str]], headers: Dict[str, str]) -> None:
     if auth is not None and "authorization" in {header.lower() for header in headers}:
         raise click.BadParameter("Passing `--auth` together with `--header` that sets `Authorization` is not allowed.")
@@ -561,11 +779,15 @@ def execute(
     store_network_log: Optional[click.utils.LazyFile],
     junit_xml: Optional[click.utils.LazyFile],
     verbosity: int,
+    code_sample_style: CodeSampleStyle,
+    debug_output_file: Optional[click.utils.LazyFile],
 ) -> None:
     """Execute a prepared runner by drawing events from it and passing to a proper handler."""
     handlers: List[EventHandler] = []
     if junit_xml is not None:
         handlers.append(JunitXMLHandler(junit_xml))
+    if debug_output_file is not None:
+        handlers.append(DebugOutputHandler(debug_output_file))
     if store_network_log is not None:
         # This handler should be first to have logs writing completed when the output handler will display statistic
         handlers.append(cassettes.CassetteWriter(store_network_log))
@@ -577,6 +799,7 @@ def execute(
         cassette_file_name=store_network_log.name if store_network_log is not None else None,
         junit_xml_file=junit_xml.name if junit_xml is not None else None,
         verbosity=verbosity,
+        code_sample_style=code_sample_style,
     )
     GLOBAL_HOOK_DISPATCHER.dispatch("after_init_cli_run_handlers", HookContext(), handlers, execution_context)
     try:
@@ -604,18 +827,23 @@ def execute(
 @click.option("--status", help="Status of interactions to replay.", type=str)
 @click.option("--uri", help="A regexp that filters interactions by their request URI.", type=str)
 @click.option("--method", help="A regexp that filters interactions by their request method.", type=str)
+@click.option("--no-color", help="Disable ANSI color escape codes.", type=bool, is_flag=True)
+@click.pass_context
 def replay(
+    ctx: click.Context,
     cassette_path: str,
     id_: Optional[str],
     status: Optional[str] = None,
     uri: Optional[str] = None,
     method: Optional[str] = None,
+    no_color: bool = False,
 ) -> None:
     """Replay a cassette.
 
     Cassettes in VCR-compatible format can be replayed.
     For example, ones that are recorded with ``store-network-log`` option of `schemathesis run` command.
     """
+    maybe_disable_color(ctx, no_color)
     click.secho(f"{bold('Replaying cassette')}: {cassette_path}")
     with open(cassette_path) as fd:
         cassette = yaml.load(fd, Loader=SafeLoader)
@@ -631,6 +859,11 @@ def bold(message: str) -> str:
     return click.style(message, bold=True)
 
 
+def maybe_disable_color(ctx: click.Context, no_color: bool) -> None:
+    if no_color or "NO_COLOR" in os.environ:
+        ctx.color = False
+
+
 @HookDispatcher.register_spec([HookScope.GLOBAL])
 def after_init_cli_run_handlers(
     context: HookContext, handlers: List[EventHandler], execution_context: ExecutionContext
