@@ -1,3 +1,4 @@
+import base64
 import os
 import platform
 import shutil
@@ -7,8 +8,8 @@ import click
 from hypothesis import settings
 
 from ..._compat import metadata
-from ...constants import __version__
-from ...models import Status
+from ...constants import CodeSampleStyle, __version__
+from ...models import Response, Status
 from ...runner import events
 from ...runner.serialization import SerializedCase, SerializedError, SerializedTestResult
 from ..context import ExecutionContext
@@ -35,7 +36,7 @@ def display_section_name(title: str, separator: str = "=", extra: str = "", **kw
 
 
 def display_subsection(result: SerializedTestResult, color: Optional[str] = "red") -> None:
-    section_name = f"{result.method}: {result.path}"
+    section_name = f"{result.method} {result.path}"
     display_section_name(section_name, "_", result.data_generation_method, fg=color)
 
 
@@ -61,7 +62,7 @@ def display_percentage(context: ExecutionContext, event: events.AfterExecution)
     styled = click.style(current_percentage, fg="cyan")
     # Total length of the message, so it will fill to the right border of the terminal.
     # Padding is already taken into account in `context.current_line_length`
-    length = get_terminal_width() - context.current_line_length + len(styled) - len(current_percentage)
+    length = max(get_terminal_width() - context.current_line_length + len(styled) - len(current_percentage), 1)
     template = f"{{:>{length}}}"
     click.echo(template.format(styled))
 
@@ -113,22 +114,25 @@ def display_errors(context: ExecutionContext, event: events.Finished) -> None:
         return
 
     display_section_name("ERRORS")
+    should_display_full_traceback_message = False
     for result in context.results:
         if not result.has_errors:
             continue
-        display_single_error(context, result)
+        should_display_full_traceback_message |= display_single_error(context, result)
     if event.generic_errors:
         display_generic_errors(context, event.generic_errors)
-    if not context.show_errors_tracebacks:
+    if should_display_full_traceback_message and not context.show_errors_tracebacks:
         click.secho(
             "Add this option to your command line parameters to see full tracebacks: --show-errors-tracebacks", fg="red"
         )
 
 
-def display_single_error(context: ExecutionContext, result: SerializedTestResult) -> None:
+def display_single_error(context: ExecutionContext, result: SerializedTestResult) -> bool:
     display_subsection(result)
+    should_display_full_traceback_message = False
     for error in result.errors:
-        _display_error(context, error, result.seed)
+        should_display_full_traceback_message |= _display_error(context, error, result.seed)
+    return should_display_full_traceback_message
 
 
 def display_generic_errors(context: ExecutionContext, errors: List[SerializedError]) -> None:
@@ -137,16 +141,27 @@ def display_generic_errors(context: ExecutionContext, errors: List[SerializedErr
         _display_error(context, error)
 
 
-def _display_error(context: ExecutionContext, error: SerializedError, seed: Optional[int] = None) -> None:
+def display_full_traceback_message(exception: str) -> bool:
+    # Some errors should not trigger the message that suggests to show full tracebacks to the user
+    return not exception.startswith("DeadlineExceeded")
+
+
+def _display_error(context: ExecutionContext, error: SerializedError, seed: Optional[int] = None) -> bool:
     if context.show_errors_tracebacks:
         message = error.exception_with_traceback
     else:
         message = error.exception
     if error.exception.startswith("InvalidSchema") and context.validate_schema:
         message += DISABLE_SCHEMA_VALIDATION_MESSAGE + "\n"
+    if error.exception.startswith("DeadlineExceeded"):
+        message += (
+            "Consider extending the deadline with the `--hypothesis-deadline` CLI option.\n"
+            "You can disable it completely with `--hypothesis-deadline=None`.\n"
+        )
     click.secho(message, fg="red")
     if error.example is not None:
         display_example(context, error.example, seed=seed)
+    return display_full_traceback_message(error.exception)
 
 
 def display_failures(context: ExecutionContext, event: events.Finished) -> None:
@@ -173,8 +188,7 @@ def display_failures_for_single_test(context: ExecutionContext, result: Serializ
             message = f"{idx}. {check.message}"
         else:
             message = None
-        example = cast(SerializedCase, check.example)  # filtered in `get_unique_failures`
-        display_example(context, example, message, result.seed)
+        display_example(context, check.example, check.response, message, result.seed)
         # Display every time except the last check
         if idx != len(checks):
             click.echo("\n")
@@ -191,6 +205,7 @@ def reduce_schema_error(message: str) -> str:
 def display_example(
     context: ExecutionContext,
     case: SerializedCase,
+    response: Optional[Response] = None,
     message: Optional[str] = None,
     seed: Optional[int] = None,
 ) -> None:
@@ -201,9 +216,14 @@ def display_example(
         click.echo()
     for line in case.text_lines:
         click.secho(line, fg="red")
-    if case.text_lines:
-        click.echo()
-    click.secho(f"Run this Python code to reproduce this failure: \n\n    {case.requests_code}\n", fg="red")
+    click.echo()
+    if response is not None and response.body is not None:
+        payload = base64.b64decode(response.body).decode(response.encoding or "utf8", errors="replace")
+        click.secho(f"----------\n\nResponse payload: `{payload}`\n", fg="red")
+    if context.code_sample_style == CodeSampleStyle.python:
+        click.secho(f"Run this Python code to reproduce this failure: \n\n    {case.requests_code}\n", fg="red")
+    if context.code_sample_style == CodeSampleStyle.curl:
+        click.secho(f"Run this cURL command to reproduce this failure: \n\n    {case.curl_code}\n", fg="red")
     if seed is not None:
         click.secho(f"Or add this option to your command line parameters: --hypothesis-seed={seed}", fg="red")
 
@@ -289,7 +309,7 @@ def display_internal_error(context: ExecutionContext, event: events.InternalErro
 
 def handle_initialized(context: ExecutionContext, event: events.Initialized) -> None:
     """Display information about the test session."""
-    context.operations_count = event.operations_count
+    context.operations_count = cast(int, event.operations_count)  # INVARIANT: should not be `None`
     display_section_name("Schemathesis test session starts")
     versions = (
         f"platform {platform.system()} -- "
@@ -310,8 +330,8 @@ def handle_initialized(context: ExecutionContext, event: events.Initialized) ->
     click.echo(f"Base URL: {event.base_url}")
     click.echo(f"Specification version: {event.specification_name}")
     click.echo(f"Workers: {context.workers_num}")
-    click.secho(f"Collected API operations: {event.operations_count}", bold=True)
-    if event.operations_count >= 1:
+    click.secho(f"Collected API operations: {context.operations_count}", bold=True)
+    if context.operations_count >= 1:
         click.echo()
 
 
