@@ -4,7 +4,7 @@ import http
 from collections import Counter
 from contextlib import contextmanager
 from copy import deepcopy
-from enum import IntEnum
+from enum import Enum
 from itertools import chain
 from logging import LogRecord
 from typing import (
@@ -20,6 +20,7 @@ from typing import (
     Optional,
     Sequence,
     Tuple,
+    Type,
     TypeVar,
     Union,
     cast,
@@ -31,19 +32,19 @@ import curlify
 import requests
 import werkzeug
 from hypothesis import event, note, reject
-from hypothesis.strategies import SearchStrategy
+from hypothesis import strategies as st
 from starlette.testclient import TestClient as ASGIClient
 
 from . import serializers
-from .constants import SERIALIZERS_SUGGESTION_MESSAGE, USER_AGENT, DataGenerationMethod
+from .constants import SERIALIZERS_SUGGESTION_MESSAGE, USER_AGENT, CodeSampleStyle, DataGenerationMethod
 from .exceptions import CheckFailed, InvalidSchema, SerializationNotPossible, get_grouped_exception
+from .hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
 from .parameters import Parameter, ParameterSet, PayloadAlternatives
 from .serializers import Serializer, SerializerContext
 from .types import Body, Cookies, FormData, Headers, NotSet, PathParameters, Query
 from .utils import NOT_SET, GenericResponse, WSGIResponse, deprecated_property, get_response_payload
 
 if TYPE_CHECKING:
-    from .hooks import HookDispatcher
     from .schemas import BaseSchema
     from .stateful import Stateful, StatefulTest
 
@@ -140,7 +141,7 @@ class Case:  # pylint: disable=too-many-public-methods
             return urlunsplit(("http", "localhost", path or "", "", ""))
         return self.base_url
 
-    def as_text_lines(self) -> List[str]:
+    def as_text_lines(self, headers: Optional[Dict[str, Any]] = None) -> List[str]:
         """Textual representation.
 
         Each component is a separate line.
@@ -152,6 +153,11 @@ class Case:  # pylint: disable=too-many-public-methods
             "Query": self.query,
             "Body": self.body,
         }
+        if headers:
+            final_headers = output["Headers"] or {}
+            final_headers = cast(Dict[str, Any], final_headers)
+            final_headers.update(headers)
+            output["Headers"] = final_headers
         max_length = max(map(len, output))
         template = f"{{:<{max_length}}} : {{}}"
 
@@ -205,7 +211,11 @@ class Case:  # pylint: disable=too-many-public-methods
             final_headers.update(headers)
             kwargs["headers"] = final_headers
         request = requests.Request(**kwargs)
-        return curlify.to_curl(request.prepare())
+        prepared = request.prepare()
+        if isinstance(prepared.body, bytes):
+            # Note, it may be not sufficient to reproduce the error :(
+            prepared.body = prepared.body.decode("utf-8", errors="replace")
+        return curlify.to_curl(prepared)
 
     def _get_base_url(self, base_url: Optional[str] = None) -> str:
         if base_url is None:
@@ -343,6 +353,7 @@ class Case:  # pylint: disable=too-many-public-methods
         response: GenericResponse,
         checks: Tuple["CheckFunction", ...] = (),
         additional_checks: Tuple["CheckFunction", ...] = (),
+        code_sample_style: Optional[str] = None,
     ) -> None:
         """Validate application response.
 
@@ -352,6 +363,7 @@ class Case:  # pylint: disable=too-many-public-methods
         :param checks: A tuple of check functions that accept ``response`` and ``case``.
         :param additional_checks: A tuple of additional checks that will be executed after ones from the ``checks``
             argument.
+        :param code_sample_style: Controls the style of code samples for failure reproduction.
         """
         from .checks import ALL_CHECKS  # pylint: disable=import-outside-toplevel
 
@@ -365,11 +377,22 @@ class Case:  # pylint: disable=too-many-public-methods
         if errors:
             exception_cls = get_grouped_exception(self.operation.verbose_name, *errors)
             formatted_errors = "\n\n".join(f"{idx}. {error.args[0]}" for idx, error in enumerate(errors, 1))
-            code = self.get_code_to_reproduce(request=response.request)
+            code_sample_style = (
+                CodeSampleStyle.from_str(code_sample_style)
+                if code_sample_style is not None
+                else self.operation.schema.code_sample_style
+            )
+            if code_sample_style == CodeSampleStyle.python:
+                code = self.get_code_to_reproduce(request=response.request)
+                code_message = f"Run this Python code to reproduce this response: \n\n    {code}\n"
+            elif code_sample_style == CodeSampleStyle.curl:
+                code = self.as_curl_command(headers=dict(response.request.headers))
+                code_message = f"Run this cURL command to reproduce this response: \n\n    {code}\n"
+            else:  # pragma: no cover
+                raise ValueError(f"Unknown code sample style: {code_sample_style.name}")
             payload = get_response_payload(response)
             raise exception_cls(
-                f"\n\n{formatted_errors}\n\n----------\n\nResponse payload: `{payload}`\n\n"
-                f"Run this Python code to reproduce this response: \n\n    {code}\n"
+                f"\n\n{formatted_errors}\n\n----------\n\nResponse payload: `{payload}`\n\n{code_message}"
             )
 
     def call_and_validate(
@@ -378,10 +401,11 @@ class Case:  # pylint: disable=too-many-public-methods
         session: Optional[requests.Session] = None,
         headers: Optional[Dict[str, Any]] = None,
         checks: Tuple["CheckFunction", ...] = (),
+        code_sample_style: Optional[str] = None,
         **kwargs: Any,
     ) -> None:
         response = self.call(base_url, session, headers, **kwargs)
-        self.validate_response(response, checks)
+        self.validate_response(response, checks, code_sample_style=code_sample_style)
 
     def get_full_url(self) -> str:
         """Make a full URL to the current API operation, including query parameters."""
@@ -415,8 +439,12 @@ def cookie_handler(client: werkzeug.Client, cookies: Optional[Cookies]) -> Gener
             client.delete_cookie("localhost", key)
 
 
-@attr.s(slots=True)  # pragma: no mutate
-class OperationDefinition:
+P = TypeVar("P", bound=Parameter)
+D = TypeVar("D")
+
+
+@attr.s  # pragma: no mutate
+class OperationDefinition(Generic[P, D]):
     """A wrapper to store not resolved API operation definitions.
 
     To prevent recursion errors we need to store definitions without resolving references. But operation definitions
@@ -424,17 +452,17 @@ class OperationDefinition:
     scope change to have a proper reference resolving later.
     """
 
-    raw: Dict[str, Any] = attr.ib()  # pragma: no mutate
-    resolved: Dict[str, Any] = attr.ib()  # pragma: no mutate
+    raw: D = attr.ib()  # pragma: no mutate
+    resolved: D = attr.ib()  # pragma: no mutate
     scope: str = attr.ib()  # pragma: no mutate
-    parameters: Sequence[Parameter] = attr.ib()  # pragma: no mutate
+    parameters: Sequence[P] = attr.ib()  # pragma: no mutate
 
 
-P = TypeVar("P", bound=Parameter)
+C = TypeVar("C", bound=Case)
 
 
-@attr.s  # pragma: no mutate
-class APIOperation(Generic[P]):
+@attr.s(eq=False)  # pragma: no mutate
+class APIOperation(Generic[P, C]):
     """A single operation defined in an API.
 
     You can get one via a ``schema`` instance.
@@ -453,6 +481,7 @@ class APIOperation(Generic[P]):
     method: str = attr.ib()  # pragma: no mutate
     definition: OperationDefinition = attr.ib(repr=False)  # pragma: no mutate
     schema: "BaseSchema" = attr.ib()  # pragma: no mutate
+    verbose_name: str = attr.ib()  # pragma: no mutate
     app: Any = attr.ib(default=None)  # pragma: no mutate
     base_url: Optional[str] = attr.ib(default=None)  # pragma: no mutate
     path_parameters: ParameterSet[P] = attr.ib(factory=ParameterSet)  # pragma: no mutate
@@ -460,10 +489,11 @@ class APIOperation(Generic[P]):
     cookies: ParameterSet[P] = attr.ib(factory=ParameterSet)  # pragma: no mutate
     query: ParameterSet[P] = attr.ib(factory=ParameterSet)  # pragma: no mutate
     body: PayloadAlternatives[P] = attr.ib(factory=PayloadAlternatives)  # pragma: no mutate
+    case_cls: Type[C] = attr.ib(default=Case)
 
-    @property
-    def verbose_name(self) -> str:
-        return f"{self.method.upper()} {self.path}"
+    @verbose_name.default
+    def _verbose_name_default(self) -> str:
+        return f"{self.method.upper()} {self.full_path}"
 
     @property
     def full_path(self) -> str:
@@ -498,11 +528,22 @@ class APIOperation(Generic[P]):
         self,
         hooks: Optional["HookDispatcher"] = None,
         data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
-    ) -> SearchStrategy:
+    ) -> st.SearchStrategy:
         """Turn this API operation into a Hypothesis strategy."""
-        return self.schema.get_case_strategy(self, hooks, data_generation_method)
+        strategy = self.schema.get_case_strategy(self, hooks, data_generation_method)
 
-    def get_strategies_from_examples(self) -> List[SearchStrategy[Case]]:
+        def _apply_hooks(dispatcher: HookDispatcher, _strategy: st.SearchStrategy[Case]) -> st.SearchStrategy[Case]:
+            for hook in dispatcher.get_all_by_name("before_generate_case"):
+                _strategy = hook(HookContext(self), _strategy)
+            return _strategy
+
+        strategy = _apply_hooks(GLOBAL_HOOK_DISPATCHER, strategy)
+        strategy = _apply_hooks(self.schema.hooks, strategy)
+        if hooks is not None:
+            strategy = _apply_hooks(hooks, strategy)
+        return strategy
+
+    def get_strategies_from_examples(self) -> List[st.SearchStrategy[Case]]:
         """Get examples from the API operation."""
         return self.schema.get_strategies_from_examples(self)
 
@@ -543,6 +584,7 @@ class APIOperation(Generic[P]):
         return self.__class__(
             path=self.path,
             method=self.method,
+            verbose_name=self.verbose_name,
             definition=self.definition,
             schema=self.schema,
             app=self.app,
@@ -562,15 +604,17 @@ class APIOperation(Generic[P]):
         cookies: Optional[Cookies] = None,
         query: Optional[Query] = None,
         body: Union[Body, NotSet] = NOT_SET,
-    ) -> Case:
+        media_type: Optional[str] = None,
+    ) -> C:
         """Create a new example for this API operation."""
-        return Case(
+        return self.case_cls(
             operation=self,
             path_parameters=path_parameters,
             headers=headers,
             cookies=cookies,
             query=query,
             body=body,
+            media_type=media_type,
         )
 
     @property
@@ -598,12 +642,12 @@ class APIOperation(Generic[P]):
 Endpoint = APIOperation
 
 
-class Status(IntEnum):
+class Status(str, Enum):
     """Status of an action or multiple actions."""
 
-    success = 1  # pragma: no mutate
-    failure = 2  # pragma: no mutate
-    error = 3  # pragma: no mutate
+    success = "success"  # pragma: no mutate
+    failure = "failure"  # pragma: no mutate
+    error = "error"  # pragma: no mutate
 
 
 @attr.s(slots=True, repr=False)  # pragma: no mutate
@@ -612,7 +656,9 @@ class Check:
 
     name: str = attr.ib()  # pragma: no mutate
     value: Status = attr.ib()  # pragma: no mutate
-    example: Optional[Case] = attr.ib(default=None)  # pragma: no mutate
+    response: GenericResponse = attr.ib()  # pragma: no mutate
+    elapsed: float = attr.ib()  # pragma: no mutate
+    example: Case = attr.ib()  # pragma: no mutate
     message: Optional[str] = attr.ib(default=None)  # pragma: no mutate
 
 
@@ -622,7 +668,7 @@ class Request:
 
     method: str = attr.ib()  # pragma: no mutate
     uri: str = attr.ib()  # pragma: no mutate
-    body: str = attr.ib()  # pragma: no mutate
+    body: Optional[str] = attr.ib()  # pragma: no mutate
     headers: Headers = attr.ib()  # pragma: no mutate
 
     @classmethod
@@ -637,7 +683,7 @@ class Request:
     @classmethod
     def from_prepared_request(cls, prepared: requests.PreparedRequest) -> "Request":
         """A prepared request version is already stored in `requests.Response`."""
-        body = prepared.body or b""
+        body = prepared.body
 
         if isinstance(body, str):
             # can be a string for `application/x-www-form-urlencoded`
@@ -650,7 +696,7 @@ class Request:
             uri=uri,
             method=method,
             headers={key: [value] for (key, value) in prepared.headers.items()},
-            body=base64.b64encode(body).decode(),
+            body=base64.b64encode(body).decode() if body is not None else body,
         )
 
 
@@ -665,8 +711,8 @@ class Response:
     status_code: int = attr.ib()  # pragma: no mutate
     message: str = attr.ib()  # pragma: no mutate
     headers: Dict[str, List[str]] = attr.ib()  # pragma: no mutate
-    body: str = attr.ib()  # pragma: no mutate
-    encoding: str = attr.ib()  # pragma: no mutate
+    body: Optional[str] = attr.ib()  # pragma: no mutate
+    encoding: Optional[str] = attr.ib()  # pragma: no mutate
     http_version: str = attr.ib()  # pragma: no mutate
     elapsed: float = attr.ib()  # pragma: no mutate
 
@@ -676,11 +722,19 @@ class Response:
         headers = {name: response.raw.headers.getlist(name) for name in response.raw.headers.keys()}
         # Similar to http.client:319 (HTTP version detection in stdlib's `http` package)
         http_version = "1.0" if response.raw.version == 10 else "1.1"
+
+        def is_empty(_response: requests.Response) -> bool:
+            # Assume the response is empty if:
+            #   - no `Content-Length` header
+            #   - no chunks when iterating over its content
+            return "Content-Length" not in headers and list(_response.iter_content()) == []
+
+        body = None if is_empty(response) else serialize_payload(response.content)
         return cls(
             status_code=response.status_code,
             message=response.reason,
-            body=serialize_payload(response.content),
-            encoding=response.encoding or "utf8",
+            body=body,
+            encoding=response.encoding,
             headers=headers,
             http_version=http_version,
             elapsed=response.elapsed.total_seconds(),
@@ -691,11 +745,19 @@ class Response:
         """Create a response from WSGI response."""
         message = http.client.responses.get(response.status_code, "UNKNOWN")
         headers = {name: response.headers.getlist(name) for name in response.headers.keys()}
+        # Note, this call ensures that `response.response` is a sequence, which is needed for comparison
+        data = response.get_data()
+        body = None if response.response == [] else serialize_payload(data)
+        encoding: Optional[str]
+        if body is not None:
+            encoding = response.mimetype_params.get("charset", response.charset)
+        else:
+            encoding = None
         return cls(
             status_code=response.status_code,
             message=message,
-            body=serialize_payload(response.data),
-            encoding=response.content_encoding or "utf-8",
+            body=body,
+            encoding=encoding,
             headers=headers,
             http_version="1.1",
             elapsed=elapsed,
@@ -754,7 +816,8 @@ class TestResult:
     logs: List[LogRecord] = attr.ib(factory=list)  # pragma: no mutate
     is_errored: bool = attr.ib(default=False)  # pragma: no mutate
     seed: Optional[int] = attr.ib(default=None)  # pragma: no mutate
-    # To show a proper reproduction code if a failure happens
+    # To show a proper reproduction code if an error happens and there is no way to get actual headers that were
+    # sent over the network. Or there could be no actual requests at all
     overridden_headers: Optional[Dict[str, Any]] = attr.ib(default=None)  # pragma: no mutate
 
     def mark_errored(self) -> None:
@@ -772,11 +835,13 @@ class TestResult:
     def has_logs(self) -> bool:
         return bool(self.logs)
 
-    def add_success(self, name: str, example: Case) -> None:
-        self.checks.append(Check(name, Status.success, example))
+    def add_success(self, name: str, example: Case, response: GenericResponse, elapsed: float) -> None:
+        self.checks.append(Check(name=name, value=Status.success, response=response, elapsed=elapsed, example=example))
 
-    def add_failure(self, name: str, example: Case, message: str) -> None:
-        self.checks.append(Check(name, Status.failure, example, message))
+    def add_failure(self, name: str, example: Case, response: GenericResponse, elapsed: float, message: str) -> None:
+        self.checks.append(
+            Check(name=name, value=Status.failure, response=response, elapsed=elapsed, example=example, message=message)
+        )
 
     def add_error(self, exception: Exception, example: Optional[Case] = None) -> None:
         self.errors.append((exception, example))
