@@ -1,3 +1,4 @@
+import threading
 import time
 from typing import Dict, List, Optional, Union
 
@@ -15,24 +16,30 @@ from .serialization import SerializedError, SerializedTestResult
 class ExecutionEvent:
     """Generic execution event."""
 
+    asdict = attr.asdict
+
 
 @attr.s(slots=True)  # pragma: no mutate
 class Initialized(ExecutionEvent):
     """Runner is initialized, settings are prepared, requests session is ready."""
 
     # Total number of operations in the schema
-    operations_count: int = attr.ib()  # pragma: no mutate
+    operations_count: Optional[int] = attr.ib()  # pragma: no mutate
+    # The place, where the API schema is located
     location: Optional[str] = attr.ib()  # pragma: no mutate
+    # The base URL against which the tests are running
     base_url: str = attr.ib()  # pragma: no mutate
+    # API schema specification name
     specification_name: str = attr.ib()  # pragma: no mutate
     # Timestamp of test run start
     start_time: float = attr.ib(factory=time.monotonic)  # pragma: no mutate
+    thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
 
     @classmethod
-    def from_schema(cls, *, schema: BaseSchema) -> "Initialized":
+    def from_schema(cls, *, schema: BaseSchema, count_operations: bool = True) -> "Initialized":
         """Computes all needed data from a schema instance."""
         return cls(
-            operations_count=schema.operations_count,
+            operations_count=schema.operations_count if count_operations else None,
             location=schema.location,
             base_url=schema.get_base_url(),
             specification_name=schema.verbose_name,
@@ -55,18 +62,27 @@ class BeforeExecution(CurrentOperationMixin, ExecutionEvent):
     It happens before a single hypothesis test, that may contain many examples inside.
     """
 
+    # HTTP method
     method: str = attr.ib()  # pragma: no mutate
+    # Full path, including the base path
     path: str = attr.ib()  # pragma: no mutate
+    # Path without the base path
     relative_path: str = attr.ib()  # pragma: no mutate
+    # The current level of recursion during stateful testing
     recursion_level: int = attr.ib()  # pragma: no mutate
+    # An unique ID which connects events that happen during testing of the same API operation
+    # It may be useful when multiple threads are involved where incoming events are not ordered
+    correlation_id: str = attr.ib()  # pragma: no mutate
+    thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
 
     @classmethod
-    def from_operation(cls, operation: APIOperation, recursion_level: int) -> "BeforeExecution":
+    def from_operation(cls, operation: APIOperation, recursion_level: int, correlation_id: str) -> "BeforeExecution":
         return cls(
             method=operation.method.upper(),
             path=operation.full_path,
             relative_path=operation.path,
             recursion_level=recursion_level,
+            correlation_id=correlation_id,
         )
 
 
@@ -83,6 +99,8 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
     result: SerializedTestResult = attr.ib()  # pragma: no mutate
     # Test running time
     elapsed_time: float = attr.ib()  # pragma: no mutate
+    correlation_id: str = attr.ib()  # pragma: no mutate
+    thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
     # Captured hypothesis stdout
     hypothesis_output: List[str] = attr.ib(factory=list)  # pragma: no mutate
 
@@ -94,6 +112,7 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
         elapsed_time: float,
         hypothesis_output: List[str],
         operation: APIOperation,
+        correlation_id: str,
     ) -> "AfterExecution":
         return cls(
             method=operation.method.upper(),
@@ -103,6 +122,7 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
             status=status,
             elapsed_time=elapsed_time,
             hypothesis_output=hypothesis_output,
+            correlation_id=correlation_id,
         )
 
 
@@ -110,6 +130,8 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
 class Interrupted(ExecutionEvent):
     """If execution was interrupted by Ctrl-C, or a received SIGTERM."""
 
+    thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
+
 
 @attr.s(slots=True)  # pragma: no mutate
 class InternalError(ExecutionEvent):
@@ -119,6 +141,7 @@ class InternalError(ExecutionEvent):
     exception_type: str = attr.ib()  # pragma: no mutate
     exception: Optional[str] = attr.ib(default=None)  # pragma: no mutate
     exception_with_traceback: Optional[str] = attr.ib(default=None)  # pragma: no mutate
+    thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
 
     @classmethod
     def from_exc(cls, exc: Exception) -> "InternalError":
@@ -164,6 +187,7 @@ class Finished(ExecutionEvent):
 
     # Total test run execution time
     running_time: float = attr.ib()  # pragma: no mutate
+    thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
 
     @classmethod
     def from_results(cls, results: TestResultSet, running_time: float) -> "Finished":
