@@ -1,6 +1,7 @@
 # pylint: disable=too-many-statements,too-many-branches
 import logging
 import time
+import uuid
 from contextlib import contextmanager
 from types import TracebackType
 from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Type, Union, cast
@@ -10,18 +11,24 @@ import attr
 import hypothesis
 import requests
 from _pytest.logging import LogCaptureHandler, catching_logs
-from hypothesis.errors import InvalidArgument
+from hypothesis.errors import HypothesisException, InvalidArgument
 from hypothesis_jsonschema._canonicalise import HypothesisRefResolutionError
 from requests.auth import HTTPDigestAuth, _basic_auth_str
 
 from ...constants import (
-    DEFAULT_DEADLINE,
     DEFAULT_STATEFUL_RECURSION_LIMIT,
     RECURSIVE_REFERENCE_ERROR_MESSAGE,
     USER_AGENT,
     DataGenerationMethod,
 )
-from ...exceptions import CheckFailed, InvalidRegularExpression, InvalidSchema, NonCheckError, get_grouped_exception
+from ...exceptions import (
+    CheckFailed,
+    DeadlineExceeded,
+    InvalidRegularExpression,
+    InvalidSchema,
+    NonCheckError,
+    get_grouped_exception,
+)
 from ...hooks import HookContext, get_all_by_name
 from ...models import APIOperation, Case, Check, CheckFunction, Status, TestResult, TestResultSet
 from ...runner import events
@@ -33,19 +40,13 @@ from ...utils import GenericResponse, Ok, WSGIResponse, capture_hypothesis_outpu
 from ..serialization import SerializedTestResult
 
 
-def get_hypothesis_settings(hypothesis_options: Dict[str, Any]) -> hypothesis.settings:
-    # Default settings, used as a parent settings object below
-    hypothesis_options.setdefault("deadline", DEFAULT_DEADLINE)
-    return hypothesis.settings(**hypothesis_options)
-
-
 @attr.s  # pragma: no mutate
 class BaseRunner:
     schema: BaseSchema = attr.ib()  # pragma: no mutate
     checks: Iterable[CheckFunction] = attr.ib()  # pragma: no mutate
     max_response_time: Optional[int] = attr.ib()  # pragma: no mutate
     targets: Iterable[Target] = attr.ib()  # pragma: no mutate
-    hypothesis_settings: hypothesis.settings = attr.ib(converter=get_hypothesis_settings)  # pragma: no mutate
+    hypothesis_settings: hypothesis.settings = attr.ib()  # pragma: no mutate
     auth: Optional[RawAuth] = attr.ib(default=None)  # pragma: no mutate
     auth_type: Optional[str] = attr.ib(default=None)  # pragma: no mutate
     headers: Optional[Dict[str, Any]] = attr.ib(default=None)  # pragma: no mutate
@@ -56,12 +57,13 @@ class BaseRunner:
     dry_run: bool = attr.ib(default=False)  # pragma: no mutate
     stateful: Optional[Stateful] = attr.ib(default=None)  # pragma: no mutate
     stateful_recursion_limit: int = attr.ib(default=DEFAULT_STATEFUL_RECURSION_LIMIT)  # pragma: no mutate
+    count_operations: bool = attr.ib(default=True)  # pragma: no mutate
 
     def execute(self) -> Generator[events.ExecutionEvent, None, None]:
         """Common logic for all runners."""
         results = TestResultSet()
 
-        initialized = events.Initialized.from_schema(schema=self.schema)
+        initialized = events.Initialized.from_schema(schema=self.schema, count_operations=self.count_operations)
         yield initialized
 
         for event in self._execute(results):
@@ -135,9 +137,13 @@ def handle_schema_error(
             data_generation_method=data_generation_method,
         )
         result.add_error(error)
-
+        correlation_id = uuid.uuid4().hex
         yield events.BeforeExecution(
-            method=method, path=error.full_path, relative_path=error.path, recursion_level=recursion_level
+            method=method,
+            path=error.full_path,
+            relative_path=error.path,
+            recursion_level=recursion_level,
+            correlation_id=correlation_id,
         )
         yield events.AfterExecution(
             method=method,
@@ -147,6 +153,7 @@ def handle_schema_error(
             result=SerializedTestResult.from_test_result(result),
             elapsed_time=0.0,
             hypothesis_output=[],
+            correlation_id=correlation_id,
         )
         results.append(result)
     else:
@@ -173,10 +180,15 @@ def run_test(  # pylint: disable=too-many-locals
         overridden_headers=headers,
         data_generation_method=data_generation_method,
     )
-    yield events.BeforeExecution.from_operation(operation=operation, recursion_level=recursion_level)
+    # To simplify connecting `before` and `after` events in external systems
+    correlation_id = uuid.uuid4().hex
+    yield events.BeforeExecution.from_operation(
+        operation=operation, recursion_level=recursion_level, correlation_id=correlation_id
+    )
     hypothesis_output: List[str] = []
     errors: List[Exception] = []
     test_start_time = time.monotonic()
+    setup_hypothesis_database_key(test, operation)
     try:
         with catch_warnings(record=True) as warnings, capture_hypothesis_output() as hypothesis_output:
             test(checks, targets, result, errors=errors, headers=headers, **kwargs)
@@ -201,8 +213,7 @@ def run_test(  # pylint: disable=too-many-locals
                 "Tests on this API operation produce unreliable results: \n"
                 "Falsified on the first call but did not on a subsequent one"
             ),
-            # Checks should not be empty, as such cases should be caught in the `NonCheckError` branch
-            result.checks[-1].example,
+            result.checks[-1].example if result.checks else None,
         )
     except hypothesis.errors.Unsatisfiable:
         # We need more clear error message here
@@ -226,6 +237,9 @@ def run_test(  # pylint: disable=too-many-locals
             result.add_error(InvalidRegularExpression(message))
         else:
             result.add_error(error)
+    except hypothesis.errors.DeadlineExceeded as error:
+        status = Status.error
+        result.add_error(DeadlineExceeded.from_exc(error))
     except Exception as error:
         status = Status.error
         result.add_error(error)
@@ -242,9 +256,23 @@ def run_test(  # pylint: disable=too-many-locals
         elapsed_time=test_elapsed_time,
         hypothesis_output=hypothesis_output,
         operation=operation,
+        correlation_id=correlation_id,
     )
 
 
+def setup_hypothesis_database_key(test: Callable, operation: APIOperation) -> None:
+    """Make Hypothesis use separate database entries for every API operation.
+
+    It increases the effectiveness of the Hypothesis database in the CLI.
+    """
+    # Hypothesis's function digest depends on the test function signature. To reflect it for the web API case,
+    # we use all API operation parameters in the digest.
+    extra = operation.verbose_name.encode("utf8")
+    for parameter in operation.definition.parameters:
+        extra += parameter.serialize().encode("utf8")
+    test.hypothesis.inner_test._hypothesis_internal_add_digest = extra  # type: ignore
+
+
 def get_invalid_regular_expression_message(warnings: List[WarningMessage]) -> Optional[str]:
     for warning in warnings:
         message = str(warning.message)
@@ -292,24 +320,35 @@ def run_checks(
         try:
             skip_check = check(response, case)
             if not skip_check:
-                result.add_success(check_name, case)
-                check_results.append(Check(check_name, Status.success, case))
+                result.add_success(check_name, case, response, elapsed_time)
+                check_results.append(
+                    Check(name=check_name, value=Status.success, response=response, elapsed=elapsed_time, example=case)
+                )
         except AssertionError as exc:
             message = str(exc)
             if not message:
                 message = f"Check '{check_name}' failed"
                 exc.args = (message,)
             errors.append(exc)
-            result.add_failure(check_name, case, message)
-            check_results.append(Check(check_name, Status.failure, case, message))
+            result.add_failure(check_name, case, response, elapsed_time, message)
+            check_results.append(
+                Check(
+                    name=check_name,
+                    value=Status.failure,
+                    response=response,
+                    elapsed=elapsed_time,
+                    example=case,
+                    message=message,
+                )
+            )
 
     if max_response_time:
         if elapsed_time > max_response_time:
             message = f"Response time exceeded the limit of {max_response_time} ms"
             errors.append(AssertionError(message))
-            result.add_failure("max_response_time", case, message)
+            result.add_failure("max_response_time", case, response, elapsed_time, message)
         else:
-            result.add_success("max_response_time", case)
+            result.add_success("max_response_time", case, response, elapsed_time)
 
     if errors:
         raise get_grouped_exception(case.operation.verbose_name, *errors)
@@ -359,6 +398,9 @@ class ErrorCollector:
         #   - The testing process is interrupted
         if not exc_type or issubclass(exc_type, CheckFailed) or not issubclass(exc_type, Exception):
             return False
+        # These exceptions are needed for control flow on the Hypothesis side. E.g. rejecting unsatisfiable examples
+        if isinstance(exc_val, HypothesisException):
+            raise
         # Exception value is not `None` and is a subclass of `Exception` at this point
         exc_val = cast(Exception, exc_val)
         self.errors.append(exc_val.with_traceback(exc_tb))
