@@ -1,3 +1,4 @@
+import json
 from typing import Any, ClassVar, Dict, Iterable, List, Tuple
 
 import attr
@@ -6,7 +7,7 @@ from ...parameters import Parameter
 from .converter import to_json_schema_recursive
 
 
-@attr.s(slots=True)
+@attr.s(slots=True, eq=False)
 class OpenAPIParameter(Parameter):
     """A single Open API operation parameter."""
 
@@ -108,7 +109,13 @@ class OpenAPIParameter(Parameter):
             if key in self.supported_jsonschema_keywords or key.startswith("x-") or key == self.nullable_field
         }
 
+    def serialize(self) -> str:
+        # For simplicity, JSON Schema semantics is not taken into account (e.g. 1 == 1.0)
+        # I.e. two semantically equal schemas may have different representation
+        return json.dumps(self.as_json_schema(), sort_keys=True)
 
+
+@attr.s(slots=True, eq=False)
 class OpenAPI20Parameter(OpenAPIParameter):
     """Open API 2.0 parameter.
 
@@ -152,6 +159,7 @@ class OpenAPI20Parameter(OpenAPIParameter):
         return None
 
 
+@attr.s(slots=True, eq=False)
 class OpenAPI30Parameter(OpenAPIParameter):
     """Open API 3.0 parameter.
 
@@ -200,7 +208,7 @@ class OpenAPI30Parameter(OpenAPIParameter):
         return super().from_open_api_to_json_schema(open_api_schema)
 
 
-@attr.s(slots=True)
+@attr.s(slots=True, eq=False)
 class OpenAPIBody(OpenAPIParameter):
     media_type: str = attr.ib()
 
@@ -214,7 +222,7 @@ class OpenAPIBody(OpenAPIParameter):
         return "body"
 
 
-@attr.s(slots=True)
+@attr.s(slots=True, eq=False)
 class OpenAPI20Body(OpenAPIBody, OpenAPI20Parameter):
     """Open API 2.0 body variant."""
 
@@ -264,7 +272,7 @@ class OpenAPI20Body(OpenAPIBody, OpenAPI20Parameter):
 FORM_MEDIA_TYPES = ("multipart/form-data", "application/x-www-form-urlencoded")
 
 
-@attr.s(slots=True)
+@attr.s(slots=True, eq=False)
 class OpenAPI30Body(OpenAPIBody, OpenAPI30Parameter):
     """Open API 3.0 body variant.
 
@@ -298,7 +306,7 @@ class OpenAPI30Body(OpenAPIBody, OpenAPI30Parameter):
         return self.required
 
 
-@attr.s(slots=True)
+@attr.s(slots=True, eq=False)
 class OpenAPI20CompositeBody(OpenAPIBody, OpenAPI20Parameter):
     """A special container to abstract over multiple `formData` parameters."""
 
