@@ -5,6 +5,7 @@ import sys
 import traceback
 import warnings
 from contextlib import contextmanager
+from inspect import getfullargspec
 from json import JSONDecodeError
 from typing import (
     Any,
@@ -25,14 +26,19 @@ from typing import (
 
 import requests
 import yaml
+import yarl
+from hypothesis.core import is_invalid_test
 from hypothesis.reporting import with_reporter
+from hypothesis.strategies import SearchStrategy
+from hypothesis.utils.conventions import InferType
 from requests.auth import HTTPDigestAuth
 from requests.exceptions import InvalidHeader  # type: ignore
 from requests.utils import check_header_validity  # type: ignore
 from werkzeug.wrappers import Response as BaseResponse
 from werkzeug.wrappers.json import JSONMixin
 
-from .types import Filter, NotSet, RawAuth
+from .constants import USER_AGENT
+from .types import Filter, GenericTest, NotSet, RawAuth
 
 try:
     from yaml import CSafeLoader as SafeLoader
@@ -276,15 +282,19 @@ def traverse_schema(schema: Schema, callback: Callable[..., Dict[str, Any]], *ar
     return schema
 
 
+def _warn_deprecation(*, thing: str, removed_in: str, replacement: str) -> None:
+    warnings.warn(
+        f"Property `{thing}` is deprecated and will be removed in Schemathesis {removed_in}. "
+        f"Use `{replacement}` instead.",
+        DeprecationWarning,
+    )
+
+
 def deprecated_property(*, removed_in: str, replacement: str) -> Callable:
     def wrapper(prop: Callable) -> Callable:
         @property  # type: ignore
         def inner(self: Any) -> Any:
-            warnings.warn(
-                f"Property `{prop.__name__}` is deprecated and will be removed in Schemathesis {removed_in}. "
-                f"Use `{replacement}` instead.",
-                DeprecationWarning,
-            )
+            _warn_deprecation(thing=prop.__name__, removed_in=removed_in, replacement=replacement)
             return prop(self)
 
         return inner
@@ -292,6 +302,29 @@ def deprecated_property(*, removed_in: str, replacement: str) -> Callable:
     return wrapper
 
 
+def deprecated(*, removed_in: str, replacement: str) -> Callable:
+    def wrapper(func: Callable) -> Callable:
+        def inner(*args: Any, **kwargs: Any) -> Any:
+            _warn_deprecation(thing=func.__name__, removed_in=removed_in, replacement=replacement)
+            return func(*args, **kwargs)
+
+        return inner
+
+    return wrapper
+
+
+def setup_headers(kwargs: Dict[str, Any]) -> None:
+    headers = kwargs.setdefault("headers", {})
+    if "user-agent" not in {header.lower() for header in headers}:
+        kwargs["headers"]["User-Agent"] = USER_AGENT
+
+
+def require_relative_url(url: str) -> None:
+    """Raise an error if the URL is not relative."""
+    if yarl.URL(url).is_absolute():
+        raise ValueError("Schema path should be relative for WSGI/ASGI loaders")
+
+
 T = TypeVar("T")
 E = TypeVar("E", bound=Exception)
 
@@ -317,3 +350,46 @@ class Err(Generic[E]):
 
 
 Result = Union[Ok[T], Err[E]]
+GivenInput = Union[SearchStrategy, InferType]
+GIVEN_ARGS_MARKER = "_schemathesis_given_args"
+GIVEN_KWARGS_MARKER = "_schemathesis_given_kwargs"
+
+
+def get_given_args(func: GenericTest) -> Tuple:
+    return getattr(func, GIVEN_ARGS_MARKER, ())
+
+
+def get_given_kwargs(func: GenericTest) -> Dict[str, Any]:
+    return getattr(func, GIVEN_KWARGS_MARKER, {})
+
+
+def is_given_applied(func: GenericTest) -> bool:
+    return hasattr(func, GIVEN_ARGS_MARKER) or hasattr(func, GIVEN_KWARGS_MARKER)
+
+
+def given_proxy(*args: GivenInput, **kwargs: GivenInput) -> Callable[[GenericTest], GenericTest]:
+    """Proxy Hypothesis strategies to ``hypothesis.given``."""
+
+    def wrapper(func: GenericTest) -> GenericTest:
+        setattr(func, GIVEN_ARGS_MARKER, args)
+        setattr(func, GIVEN_KWARGS_MARKER, kwargs)
+        return func
+
+    return wrapper
+
+
+def merge_given_args(func: GenericTest, args: Tuple, kwargs: Dict[str, Any]) -> Dict[str, Any]:
+    """Merge positional arguments to ``@schema.given`` into a dictionary with keyword arguments.
+
+    Kwargs are modified inplace.
+    """
+    if args:
+        argspec = getfullargspec(func)
+        for name, strategy in zip(reversed([arg for arg in argspec.args if arg != "case"]), reversed(args)):
+            kwargs[name] = strategy
+    return kwargs
+
+
+def validate_given_args(func: GenericTest, args: Tuple, kwargs: Dict[str, Any]) -> Optional[Callable]:
+    argspec = getfullargspec(func)
+    return is_invalid_test(func.__name__, argspec, args, kwargs)  # type: ignore
